/*
 * Copyright (C) 2012-2013 jeanfi@gmail.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>

#include <sys/stat.h>
#include <sys/types.h>

#include <json/json.h>
#include <webkit/webkit.h>
#include <glib/gi18n.h>
#include <gtk/gtk.h>

#include "http.h"
#include "log.h"
#include "ttrss.h"
#include "ttrss_ws.h"
#include "webbrowser.h"

static const char *program_name;
static GSettings *settings;

static GtkPaned *w_panel;

static int model_state;

enum {
	COL_FEED_TITLE,
	COL_FEED_ID
};

enum {
	COL_HEADLINE_TITLE,
	COL_HEADLINE_ID
};

static struct option long_options[] = {
	{"version", no_argument, 0, 'v'},
	{"help", no_argument, 0, 'h'},
	{"debug", required_argument, 0, 'd'},
	{0, 0, 0, 0}
};

static void print_version()
{
	printf("prss %s\n", VERSION);
	printf(_("Copyright (C) %s jeanfi@gmail.com\n"
		 "License GPLv2: GNU GPL version 2 or later "
		 "<http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>\n"
		 "This is free software: you are free to change and "
		 " redistribute it.\n"
		 "There is NO WARRANTY, to the extent permitted by law.\n"),
	       "2012-2013");
}

static void print_help()
{
	printf(_("Usage: %s [OPTION]...\n"), program_name);

	puts(_("Prss is a GTK+ news reader for tt-rss."));

	puts("");
	puts(_("Options:"));
	puts(_("  -h, --help          display this help and exit\n"
	       "  -v, --version       display version information and exit"));
	puts("");
	puts(_("  -d, --debug=LEVEL   "
	       "set the debug level, integer between 0 and 3"));

	puts("");

	printf(_("Report bugs to: %s\n"), PACKAGE_BUGREPORT);
	puts("");
	printf(_("%s home page: <%s>\n"), PACKAGE_NAME, PACKAGE_URL);
}

GtkTreeView *w_feedtree;
GtkTreeView *w_headlinetree;

void update()
{
	struct feed **feeds;
	GtkTreeModel *model, *headlinemodel;
	GtkTreeIter iter;
	char *title;

	model = gtk_tree_view_get_model(GTK_TREE_VIEW(w_feedtree));

	log_debug("update(): clear feed tree.");
	model_state = 1;
	gtk_list_store_clear(GTK_LIST_STORE(model));
	headlinemodel = gtk_tree_view_get_model(w_headlinetree);
	gtk_list_store_clear(GTK_LIST_STORE(headlinemodel));
	log_debug("update(): clear feed tree done.");
	feeds = ttrss_get_feeds();
	while (feeds && *feeds) {
		if ((*feeds)->unread)
			title = g_strdup_printf("<b>%s (%d)</b>",
						(*feeds)->title,
						(*feeds)->unread);
		else
			title = strdup((*feeds)->title);


		gtk_list_store_append(GTK_LIST_STORE(model), &iter);

		gtk_list_store_set(GTK_LIST_STORE(model),
				   &iter,
				   COL_FEED_TITLE, title,
				   COL_FEED_ID, (*feeds)->id,
				   -1);
		free(title);
		feeds++;
	}
	model_state = 0;

	log_debug("update() done");
}

void update_layout()
{
	int layout;

	layout = g_settings_get_int(settings, "layout");

	switch (layout) {
	case 1:
		gtk_orientable_set_orientation(GTK_ORIENTABLE(w_panel),
					       GTK_ORIENTATION_HORIZONTAL);
		break;
	case 0:
	default:
		gtk_orientable_set_orientation(GTK_ORIENTABLE(w_panel),
					       GTK_ORIENTATION_VERTICAL);
	}
}

void refresh_clicked_cbk(GtkWidget *btn, gpointer data)
{
	log_debug("refresh_clicked_cbk()");
	update();
	log_debug("refresh_clicked_cbk() done");
}

void settings_activate_cbk(GtkWidget *menu_item, gpointer data)
{
	GtkDialog *diag;
	GtkBuilder *builder;
	GtkEntry *login, *url, *pwd;
	GtkComboBox *layout;

	log_debug("settings_activate_cbk()");

	builder = gtk_builder_new();

	gtk_builder_add_from_file
		(builder,
		 PACKAGE_DATA_DIR G_DIR_SEPARATOR_S "prss.glade",
		 NULL);

	diag = GTK_DIALOG(gtk_builder_get_object(builder, "settings_dialog"));

	url = GTK_ENTRY(gtk_builder_get_object(builder, "url_entry"));
	gtk_entry_set_text(url, g_settings_get_string(settings, "url"));

	login = GTK_ENTRY(gtk_builder_get_object(builder, "login_entry"));
	gtk_entry_set_text(login, g_settings_get_string(settings, "user"));

	pwd = GTK_ENTRY(gtk_builder_get_object(builder, "password_entry"));
	gtk_entry_set_text(pwd, g_settings_get_string(settings, "password"));

	layout = GTK_COMBO_BOX(gtk_builder_get_object(builder,
						      "layout"));

	gtk_combo_box_set_active(layout, g_settings_get_int(settings,
							    "layout"));

	if (gtk_dialog_run(diag)) {
		g_settings_set_int(settings,
				   "layout",
				   gtk_combo_box_get_active(layout));

		g_settings_set_string(settings,
				      "password",
				      gtk_entry_get_text(pwd));
		g_settings_set_string(settings,
				      "url",
				      gtk_entry_get_text(url));
		g_settings_set_string(settings,
				      "user",
				      gtk_entry_get_text(login));

		ttrss_set_config(g_settings_get_string(settings, "url"),
				 g_settings_get_string(settings, "user"),
				 g_settings_get_string(settings, "password"));
		ws_open_session();

		update_layout();
		update();
	}

	gtk_widget_destroy(GTK_WIDGET(diag));

	g_object_unref(G_OBJECT(builder));
}

static struct feed *get_selected_feed()
{
	GtkTreePath *path;
	GtkTreeViewColumn *cols;
	GtkTreeIter iter;
	GtkTreeModel *model;
	int id;

	gtk_tree_view_get_cursor(w_feedtree, &path, &cols);

	if (path) {
		model = gtk_tree_view_get_model(w_feedtree);
		gtk_tree_model_get_iter(model, &iter, path);
		gtk_tree_model_get(model, &iter, 1, &id, -1);

		gtk_tree_path_free(path);

		return ttrss_get_feed(id);
	}

	return NULL;
}

static struct headline *get_selected_headline(GtkTreeIter *iter)
{
	GtkTreePath *path;
	GtkTreeViewColumn *cols;
	GtkTreeModel *model;
	int id;

	gtk_tree_view_get_cursor(w_headlinetree, &path, &cols);

	if (path) {
		model = gtk_tree_view_get_model(w_headlinetree);
		gtk_tree_model_get_iter(model, iter, path);
		gtk_tree_model_get(model, iter, 1, &id, -1);

		gtk_tree_path_free(path);

		return ttrss_get_headline(id);
	}

	return NULL;
}

int feed_cursor_changed_cbk(GtkTreeView *treeview, gpointer data)
{
	GtkTreeIter iter;
	GtkTreeModel *headline_model;
	GtkListStore *headline_store;
	struct feed *feed;
	struct headline **headlines;
	char *title;

	if (model_state)
		return TRUE;

	log_debug("feed_cursor_changed_cbk()");

	feed = get_selected_feed();

	if (feed) {
		headline_model = gtk_tree_view_get_model(w_headlinetree);
		headline_store = GTK_LIST_STORE(headline_model);
		model_state = 1;
		gtk_list_store_clear(headline_store);

		if (feed) {
			headlines = ttrss_feed_get_headlines(feed);
			while (headlines && *headlines) {
				gtk_list_store_append(headline_store, &iter);

				if ((*headlines)->unread)
					title = g_strdup_printf
						("<b>%s</b>",
						 (*headlines)->title);
				else
					title = strdup((*headlines)->title);

				gtk_list_store_set(headline_store,
						   &iter,
						   COL_HEADLINE_TITLE,
						   title,
						   COL_HEADLINE_ID,
						   (*headlines)->id,
						   -1);

				free(title);

				headlines++;
			}
		}

		model_state = 0;
	}

	return 1;
}

int headline_cursor_changed_cbk(GtkTreeView *treeview, gpointer data)
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	struct headline *headline;
	char *str, *tmp;

	log_debug("headline_cursor_changed_cbk()");

	if (model_state)
		return TRUE;

	headline = get_selected_headline(&iter);

	if (headline) {
		tmp = ttrss_get_headline_content(headline);

		if (tmp) {
			str = g_strdup_printf("<h1><a href='%s'>%s</a></h1>%s",
					      headline->url,
					      headline->title,
					      tmp);
			free(tmp);
			
			web_load(str);
			free(str);
		} else {
			web_load("");
		}

		
		if (headline->unread) {
			ttrss_set_article_unread(headline->id, 0);
			headline->unread = 0;
		}

		model = gtk_tree_view_get_model(w_headlinetree);
		gtk_list_store_set(GTK_LIST_STORE(model),
				   &iter,
				   0, headline->title,
				   -1);
	} else {
		log_debug("headline not found");
	}

	return TRUE;
}

void headline_row_activated_cbk(GtkWidget *widget, gpointer data)
{
	GdkScreen *screen;
	GError *error;
	GtkTreeSelection *selection;
	GtkTreeModel *model;
	GtkTreeIter iter;
	struct headline *h;
	const char *url;
	int id;

	printf("headline_row_activated_cbk()");

	selection = gtk_tree_view_get_selection(w_headlinetree);
	model = gtk_tree_view_get_model(w_headlinetree);

	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter, 1, &id, -1);

		h = ttrss_get_headline(id);
		if (h) {
			url = h->url;

			if (gtk_widget_has_screen(widget))
				screen = gtk_widget_get_screen(widget);
			else
				screen = gdk_screen_get_default();

			error = NULL;
			gtk_show_uri(screen, url,
				     gtk_get_current_event_time(), &error);
		}
	}
}

gboolean
window_delete_event_cbk(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	log_debug("window_delete_event_cbk");

	gtk_main_quit();

	return TRUE;
}

void quit_activate_cbk(GtkWidget *menu_item, gpointer data)
{
	log_debug("quit_activate_cbk");

	gtk_main_quit();
}

void help_activate_cbk(GtkWidget *menu_item, gpointer data)
{
	log_debug("help_activate_cbk");

	gtk_show_about_dialog
		(NULL,
		 "comments",
		 _("PRSS is a GTK+ client application for Tiny Tiny RSS"),
		 "copyright",
		 _("Copyright(c) 2010-2013\njeanfi@gmail.com"),
		 "logo-icon-name", "prss",
		 "program-name", "PRSS",
		 "title", _("About PRSS"),
		 "version", VERSION,
		 "website", PACKAGE_URL,
		 "website-label", _("PRSS Homepage"),
		 NULL);
}

static void log_init()
{
	char *home, *path, *dir;

	home = getenv("HOME");

	if (!home)
		return ;

	dir = malloc(strlen(home)+1+strlen(".prss")+1);
	sprintf(dir, "%s/%s", home, ".prss");
	mkdir(dir, 0777);

	path = malloc(strlen(dir)+1+strlen("log")+1);
	sprintf(path, "%s/%s", dir, "log");

	log_open(path);

	free(dir);
	free(path);
}

int main(int argc, char **argv)
{
	GtkWidget *window, *panh;
	GtkBuilder *builder;
	int optc, cmdok, opti;

	program_name = argv[0];

	setlocale(LC_ALL, "");

#if ENABLE_NLS
	bindtextdomain(PACKAGE, LOCALEDIR);
	textdomain(PACKAGE);
#endif

	cmdok = 1;
	while ((optc = getopt_long(argc, argv, "vhd:", long_options,
				   &opti)) != -1) {
		switch (optc) {
		case 'd':
			log_level = atoi(optarg);
			log_info(_("Enables debug mode."));
			break;
		case 'h':
			print_help();
			exit(EXIT_SUCCESS);
		case 'v':
			print_version();
			exit(EXIT_SUCCESS);
		default:
			cmdok = 0;
			break;
		}
	}

	if (!cmdok || optind != argc) {
		fprintf(stderr, _("Try `%s --help' for more information.\n"),
			program_name);
		exit(EXIT_FAILURE);
	}

	log_init();

	http_init();
	ws_init();

	gtk_init(NULL, NULL);
	settings = g_settings_new("prss");

	ttrss_set_config(g_settings_get_string(settings, "url"),
			 g_settings_get_string(settings, "user"),
			 g_settings_get_string(settings, "password"));
	ws_open_session();

	builder = gtk_builder_new();
	gtk_builder_add_from_file
		(builder,
		 PACKAGE_DATA_DIR G_DIR_SEPARATOR_S "prss.glade",
		 NULL);
	window = GTK_WIDGET(gtk_builder_get_object(builder, "window"));
	w_feedtree = GTK_TREE_VIEW(gtk_builder_get_object(builder,
							  "feed_tree"));
	w_headlinetree = GTK_TREE_VIEW(gtk_builder_get_object(builder,
							      "headline_tree"));

	w_panel = GTK_PANED(gtk_builder_get_object(builder, "paned2"));

	gtk_builder_connect_signals(builder, NULL);

	panh = GTK_WIDGET(gtk_builder_get_object(builder, "webkit_window"));

	gtk_container_add(GTK_CONTAINER(panh), web_get_widget());

	g_object_unref(G_OBJECT(builder));

	gtk_widget_show_all(window);
	update_layout();

	gtk_main();

	exit(EXIT_SUCCESS);
}
